@@ -12,16 +12,6 @@ from __future__ import absolute_import, division, unicode_literals
 import re
 import sys
 
-from mo_future import (
-    binary_type,
-    generator_types,
-    text,
-    OrderedDict,
-    none_type, flatten,
-    first,
-)
-from mo_imports import export
-
 from mo_dots.datas import Data, data_types, is_data
 from mo_dots.lists import (
     FlatList,
@@ -32,10 +22,20 @@ from mo_dots.lists import (
     list_types,
     container_types,
     finite_types,
+    last
 )
 from mo_dots.nones import Null, NullType
 from mo_dots.objects import DataObject
 from mo_dots.utils import CLASS, SLOT, get_logger, get_module
+from mo_future import (
+    binary_type,
+    generator_types,
+    text,
+    OrderedDict,
+    none_type, flatten,
+    first,
+)
+from mo_imports import export
 
 _module_type = type(sys.modules[__name__])
 _builtin_zip = zip
@@ -44,8 +44,6 @@ _set = object.__setattr__
 _new = object.__new__
 _dict_zip = zip
 
-ROOT_PATH = ["."]
-
 
 def inverse(d):
     """
@@ -208,7 +206,24 @@ def startswith_field(field, prefix):
         #     return True
 
     if field.startswith(prefix):
-        if len(field) == len(prefix) or field[len(prefix)] == ".":
+        lp = len(prefix)
+        if len(field) == len(prefix) or field[lp] in (".", "\b") and field[lp+1] not in (".", "\b"):
+            return True
+    return False
+
+
+def endswith_field(field, suffix):
+    """
+    RETURN True IF field PATH STRING ENDS WITH suffix PATH STRING
+    """
+    if suffix == None:
+        return False
+    if suffix == ".":
+        return True
+
+    if field.endswith(suffix):
+        ls = len(suffix)
+        if len(field) == ls or field[-ls - 1] in (".", "\b") and field[-ls - 2] not in (".", "\b"):
             return True
     return False
 
@@ -266,7 +281,7 @@ def set_default(d, *dicts):
     """
     agg = d if d or _get(d, CLASS) in data_types else {}
     for p in dicts:
-        _set_default(agg, from_data(p), seen={})
+        _set_default(agg, p, seen={})
     return to_data(agg)
 
 
@@ -279,7 +294,7 @@ def _set_default(d, default, seen=None):
         return
 
     for k, default_value in default.items():
-        default_value = from_data(default_value)  # TWO DIFFERENT Dicts CAN SHARE id() BECAUSE THEY ARE SHORT LIVED
+        raw_value = from_data(default_value)  # TWO DIFFERENT Dicts CAN SHARE id() BECAUSE THEY ARE SHORT LIVED
         if is_data(d):
             existing_value = d.get(k)
         else:
@@ -288,12 +303,12 @@ def _set_default(d, default, seen=None):
         if existing_value == None:
             if default_value != None:
                 if _get(default_value, CLASS) in data_types:
-                    df = seen.get(id(default_value))
+                    df = seen.get(id(raw_value))
                     if df is not None:
                         _set_attr(d, [k], df)
                     else:
                         copy_dict = {}
-                        seen[id(default_value)] = copy_dict
+                        seen[id(raw_value)] = copy_dict
                         _set_attr(d, [k], copy_dict)
                         _set_default(copy_dict, default_value, seen)
                 else:
@@ -312,11 +327,11 @@ def _set_default(d, default, seen=None):
             hasattr(existing_value, "__setattr__")
             or _get(existing_value, CLASS) in data_types
         ) and _get(default_value, CLASS) in data_types:
-            df = seen.get(id(default_value))
+            df = seen.get(id(raw_value))
             if df is not None:
                 _set_attr(d, [k], df)
             else:
-                seen[id(default_value)] = existing_value
+                seen[id(raw_value)] = existing_value
                 _set_default(existing_value, default_value, seen)
 
 
@@ -524,7 +539,7 @@ def list_to_data(v):
     return output
 
 
-def to_data(v=None):
+def to_data(v=None) -> object:
     """
     WRAP AS Data OBJECT FOR DATA PROCESSING: https://github.com/klahnakoski/mo-dots/tree/dev/docs
     :param v:  THE VALUE TO WRAP
@@ -539,6 +554,8 @@ def to_data(v=None):
         return m
     elif type_ is none_type:
         return Null
+    elif type_ is tuple:
+        return list_to_data(v)
     elif type_ is list:
         return list_to_data(v)
     elif type_ in generator_types:
@@ -685,7 +702,7 @@ def tuplewrap(value):
     """
     INTENDED TO TURN lists INTO tuples FOR USE AS KEYS
     """
-    if value==None:
+    if value == None:
         return tuple()
     elif is_many(value):
         return tuple(tuplewrap(v) if is_sequence(v) else v for v in value)
@@ -744,7 +761,9 @@ export("mo_dots.lists", from_data)
 export("mo_dots.lists", hash_value)
 
 export("mo_dots.objects", list_to_data)
+export("mo_dots.objects", dict_to_data)
 export("mo_dots.objects", to_data)
 export("mo_dots.objects", from_data)
 export("mo_dots.objects", get_attr)
 export("mo_dots.objects", set_attr)
+export("mo_dots.objects", set_default)
